package se.umu.cs.edu.soa.directoryservice;

import java.io.*;

public class DirectoryServiceCore
{
  private final EntryAccessor accessor;


  //---------------------------------------------------------
  private DirectoryServiceCore (EntryAccessor accessor)
  {
    this.accessor = accessor;
  }

  //---------------------------------------------------------
  public EntryType[] retrieveEntrys (String[] types)
    throws IOException
  {
    return accessor.retrieveEntrys(types);
  }

  //---------------------------------------------------------
  public void storeEntrys (EntryType[] entrys)
    throws IOException
  {
    accessor.storeEntrys(entrys);
  }

  //---------------------------------------------------------
  public void deleteEntrys (String[] ids)
    throws IOException
  {
    accessor.deleteEntrys(ids);
  }


  //---------------------------------------------------------
  //---------------------------------------------------------
  public static class Factory
  {
    private static final String PROPERTY_ACCESSOR = "directoryservice.accessor";
    private static final DirectoryServiceCore SINGLETON =
      new DirectoryServiceCore(getAccessorInstance());


    //---------------------------------------------------------
    //---------------------------------------------------------
    private static EntryAccessor getAccessorInstance ()
    {
      try
      {
        String accessor = System.getProperty(PROPERTY_ACCESSOR);
        if (accessor != null)
          return (EntryAccessor)Class.forName(accessor).newInstance();
        else
          return new EntryAccessorSimulator();
      }
      catch (Exception e)
      {
        throw new IllegalArgumentException();
      }
    }

    //---------------------------------------------------------
    public static DirectoryServiceCore getInstance ()
    {
      return SINGLETON;
    }
  }
}
