package se.umu.cs.edu.soa.directoryservice;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Set;

public class EntryAccessorSimulator
  implements EntryAccessor
{
  private final HashMap<String,ArrayList<EntryType>> entryMap;


  //---------------------------------------------------------
  public EntryAccessorSimulator ()
  {
    entryMap = new HashMap<String,ArrayList<EntryType>>();

/*
    // test / debug
    Entry[] entrys = new Entry[]
    {
      new Entry("1","ChatService",
                "http://localhost:8080/axis2/services/ChatService",
                "A chat service, used for information end-user communication"),
      new Entry("2","LogService",
                "http://localhost:8080/axis2/services/LogService",
                "A log service, used for logging chat messages"),
      new Entry("3","MonitorService",
                "http://localhost:8080/axis2/services/MonitorService",
                "A monitor service, used for content-based filtering of chats"),
    };
    try
    {
      storeEntrys(entrys);
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
*/
  }


  //---------------------------------------------------------
  //---------------------------------------------------------
  // EntryAccessor interface

  //---------------------------------------------------------
  public EntryType[] retrieveEntrys (String[] types)
    throws IOException
  {
    ArrayList<EntryType> entryList = new ArrayList<EntryType>();
    synchronized (entryMap)
    {
      for (String type : types)
      {
        ArrayList<EntryType> topicEntryList = entryMap.get(type);
        if(topicEntryList != null) {
        	for (EntryType entry : topicEntryList) {
        		entryList.add(entry);
        	}
        }
      }
    }   
    
    EntryType[] rval = new EntryType[entryList.size()];
    entryList.toArray(rval);
    return rval;
  }

  //---------------------------------------------------------
  public void storeEntrys (EntryType[] entrys)
    throws IOException
  {
		
    synchronized (entryMap)
    {
      for (EntryType entry : entrys)
      {
        ArrayList<EntryType> topicEntryList = entryMap.get(entry.getType().localServiceType);
        if (topicEntryList == null)
        {
          topicEntryList = new ArrayList<EntryType>();
          entryMap.put(entry.getType().localServiceType,topicEntryList);
          
        }        
    
        topicEntryList.add(entry);                      
   
      }
    }
  }

  //---------------------------------------------------------
  private static boolean member (String value, String[] values)
  {
    for (String v : values)
      if (v.equals(value))
        return true;
    return false;
  }

  //---------------------------------------------------------
  public void deleteEntrys (String[] ids)
    throws IOException
  {
    synchronized (entryMap)
    {
      Set<String> keySet = entryMap.keySet();
      for (String type : keySet)
      {
        ArrayList<EntryType> topicEntryList = entryMap.get(type);
        ArrayList<EntryType> removeList = new ArrayList<EntryType>();
        for (EntryType entry : topicEntryList)
          if (member(entry.getId().toString(),ids))
            removeList.add(entry);
        for (EntryType entry : removeList)
          topicEntryList.remove(entry);
      }
    }
  }
}
